/// <reference types="node" />
import Tunnel, { TunnelProperties, DownloadOptions, ChildExecutor, NormalizedEnvironment } from './Tunnel';
import { JobState } from './interfaces';
import Task from '@dojo/core/async/Task';
import { Url } from 'url';
/**
 * A Sauce Labs tunnel. This tunnel uses Sauce Connect 4 on platforms where it is supported, and Sauce Connect 3 on all
 * other platforms.
 *
 * The accessKey and username properties will be initialized using SAUCE_ACCESS_KEY and SAUCE_USERNAME.
 */
export default class SauceLabsTunnel extends Tunnel implements SauceLabsProperties {
    /** A list of domains that should not be proxied by the tunnel on the remote VM. */
    directDomains: string[];
    /** A list of domains that will be proxied by the tunnel on the remote VM. */
    tunnelDomains: string[];
    /**
     * A list of URLs that require additional HTTP authentication. Only the hostname, port, and auth are used.
     * This property is only supported by Sauce Connect 4 tunnels.
     */
    domainAuthentication: string[];
    /**
     * A list of regular expressions corresponding to domains whose connections should fail immediately if the VM
     * attempts to make a connection to them.
     */
    fastFailDomains: string[];
    /** Allows the tunnel to also be used by sub-accounts of the user that started the tunnel. */
    isSharedTunnel: boolean;
    /** A filename where additional logs from the tunnel should be output. */
    logFile: string;
    /** A filename where Sauce Connect stores its process information. */
    pidFile: string;
    /**
     * Specifies the maximum log filesize before rotation, in bytes. This property is only supported by Sauce Connect 3
     * tunnels.
     */
    logFileSize: number;
    /**
     * Log statistics about HTTP traffic every `logTrafficStats` milliseconds. This property is only supported by Sauce
     * Connect 4 tunnels.
     */
    logTrafficStats: number;
    /**
     * An alternative URL for the Sauce REST API. This property is only supported by Sauce Connect 3 tunnels.
     */
    restUrl: string;
    /**
     * A list of domains that should not have their SSL connections re-encrypted when going through the tunnel.
     */
    skipSslDomains: string[];
    /**
     * An additional set of options to use with the Squid proxy for the remote VM. This property is only supported by
     * Sauce Connect 3 tunnels.
     */
    squidOptions: string;
    /**
     * Whether or not to use the proxy defined at {@link module:digdug/Tunnel#proxy} for the tunnel connection itself.
     */
    useProxyForTunnel: boolean;
    /**
     * Overrides the version of the VM created on Sauce Labs. This property is only supported by Sauce Connect 3
     * tunnels.
     */
    vmVersion: string;
    scVersion: string;
    constructor(options?: SauceLabsOptions);
    readonly auth: string;
    readonly executable: string;
    readonly extraCapabilities: any;
    readonly isDownloaded: boolean;
    readonly url: string;
    protected _postDownloadFile(data: Buffer, options?: DownloadOptions): Promise<void>;
    protected _makeNativeArgs(proxy?: Url): string[];
    protected _makeJavaArgs(proxy?: Url): string[];
    protected _makeArgs(readyFile: string): string[];
    sendJobState(jobId: string, data: JobState): Task<void>;
    protected _start(executor: ChildExecutor): Task<any>;
    /**
     * Attempt to normalize a SauceLabs described environment with the standard Selenium capabilities
     *
     * SauceLabs returns a list of environments that looks like:
     *
     * {
     *     "short_version": "25",
     *     "long_name": "Firefox",
     *     "api_name": "firefox",
     *     "long_version": "25.0b2.",
     *     "latest_stable_version": "",
     *     "automation_backend": "webdriver",
     *     "os": "Windows 2003"
     * }
     *
     * @param environment a SauceLabs environment descriptor
     * @returns a normalized descriptor
     */
    protected _normalizeEnvironment(environment: any): NormalizedEnvironment;
}
export interface SauceLabsProperties extends TunnelProperties {
    directDomains: string[];
    tunnelDomains: string[];
    domainAuthentication: string[];
    fastFailDomains: string[];
    isSharedTunnel: boolean;
    logFile: string;
    pidFile: string;
    logFileSize: number;
    logTrafficStats: number;
    restUrl: string;
    skipSslDomains: string[];
    squidOptions: string;
    useProxyForTunnel: boolean;
    vmVersion: string;
}
export declare type SauceLabsOptions = Partial<SauceLabsProperties>;
