/// <reference types="node" />
import Tunnel, { TunnelProperties, DownloadOptions, ChildExecutor } from './Tunnel';
import Task from '@dojo/core/async/Task';
/**
 * A Selenium tunnel. This tunnel downloads the {@link http://www.seleniumhq.org/download/ Selenium-standalone server}
 * and any necessary WebDriver executables, and handles starting and stopping Selenium.
 *
 * The primary configuration option is {@linkcode module:digdug/SeleniumTunnel#drivers drivers}, which determines which
 * browsers the Selenium tunnel will support.
 *
 * Note that Java must be installed and in the system path to use this tunnel.
 */
export default class SeleniumTunnel extends Tunnel implements SeleniumProperties {
    /** Additional arguments to send to the Selenium server at startup */
    seleniumArgs: string[];
    /**
     * The desired Selenium drivers to install. Each entry may be a string or an object. Strings must be the names of
     * existing drivers in SeleniumTunnel. An object with a 'name' property is a configuration object -- the name must
     * be the name of an existing driver in SeleniumTunnel, and the remaining properties will be used to configure that
     * driver. An object without a 'name' property is a driver definition. It must contain three properties:
     *
     *   - executable - the name of the driver executable
     *   - url - the URL where the driver can be downloaded from
     *   - seleniumProperty - the name of the Java property used to tell Selenium where the driver is
     *
     * @example
     * 	[
     *      'chrome',
     *      {
     *          name: 'firefox',
     *          version: '0.8.0'
     *      },
     *      {
     *          url: 'https://github.com/operasoftware/operachromiumdriver/releases/.../operadriver_mac64.zip',
     *          executable: 'operadriver',
     *          seleniumProperty: 'webdriver.opera.driver'
     *      }
     * 	]
     *
     * @default [ 'chrome' ]
     */
    drivers: DriverDescriptor[];
    /**
     * The base address where Selenium artifacts may be found.
     *
     * @default https://selenium-release.storage.googleapis.com
     */
    baseUrl: string;
    /**
     * The desired version of selenium to install.
     *
     * @default 3.3.1
     */
    version: string;
    /**
     * Timeout in milliseconds for communicating with the Selenium server
     *
     * @default 5000
     */
    seleniumTimeout: number;
    constructor(options?: SeleniumOptions);
    readonly artifact: string;
    readonly directory: string;
    readonly executable: string;
    readonly isDownloaded: boolean;
    readonly url: string;
    download(forceDownload?: boolean): Task<any>;
    sendJobState(): Task<void>;
    protected _getDriverConfigs(): DriverFile[];
    protected _makeArgs(): string[];
    protected _postDownloadFile(data: Buffer, options: SeleniumDownloadOptions): Promise<void>;
    protected _start(executor: ChildExecutor): Task<any>;
}
export interface DriverFile extends RemoteFile {
    seleniumProperty: string;
}
export interface RemoteFile {
    dontExtract?: boolean;
    executable: string;
    url: string;
}
export declare type DriverDescriptor = 'chrome' | 'ie' | 'firefox' | DriverFile | {
    name: string;
};
export interface SeleniumProperties extends TunnelProperties {
    seleniumArgs: string[];
    drivers: DriverDescriptor[];
    baseUrl: string;
    version: string;
    seleniumTimeout: number;
}
export declare type SeleniumOptions = Partial<SeleniumProperties>;
export interface SeleniumDownloadOptions extends DownloadOptions {
    executable?: string;
}
