/// <reference types="node" />
import Evented, { BaseEventedEvents } from '@dojo/core/Evented';
import { EventObject, EventTargettedObject, Handle } from '@dojo/interfaces/core';
import Task from '@dojo/core/async/Task';
import { ChildProcess } from 'child_process';
import { Url } from 'url';
import { JobState } from './interfaces';
/**
 * A Tunnel is a mechanism for connecting to a WebDriver service provider that securely exposes local services for
 * testing within the service provider’s network.
 */
export default class Tunnel extends Evented implements TunnelProperties, Url {
    constructor(options?: TunnelOptions);
    /**
     * Part of the tunnel has been downloaded from the server.
     *
     * @event module:digdug/Tunnel#downloadprogress
     * @type {Object}
     * @property {number} received The number of bytes received so far.
     * @property {number} total The total number of bytes to download.
     */
    /**
     * A chunk of raw string data output by the tunnel software to stdout.
     *
     * @event module:digdug/Tunnel#stdout
     * @type {string}
     */
    /**
     * A chunk of raw string data output by the tunnel software to stderr.
     *
     * @event module:digdug/Tunnel#stderr
     * @type {string}
     */
    /**
     * Information about the status of the tunnel setup process that is suitable for presentation to end-users.
     *
     * @event module:digdug/Tunnel#status
     * @type {string}
     */
    on: Events;
    /** The URL of a service that provides a list of environments supported by the tunnel. */
    environmentUrl: string;
    /**
     * The tunnel access key. This will be initialized with a tunnel-specific environment variable if not specified.
     */
    accessKey: string;
    /**
     * The tunnel username. This will be initialized with a tunnel-specific environment variable if not specified.
     */
    username: string;
    /**
     * The architecture the tunnel will run against. This information is automatically retrieved for the current
     * system at runtime.
     */
    architecture: string;
    /**
     * An HTTP authorization string to use when initiating connections to the tunnel. This value of this property is
     * defined by Tunnel subclasses.
     */
    auth: string;
    /**
     * The directory where the tunnel software will be extracted. If the directory does not exist, it will be
     * created. This value is set by the tunnel subclasses.
     */
    directory: string;
    /** The executable to spawn in order to create a tunnel. This value is set by the tunnel subclasses. */
    executable: string;
    /**
     * The host on which a WebDriver client can access the service provided by the tunnel. This may or may not be
     * the host where the tunnel application is running.
     */
    hostname: string;
    /** The path that a WebDriver client should use to access the service provided by the tunnel. */
    pathname: string;
    /**
     * The operating system the tunnel will run on. This information is automatically retrieved for the current
     * system at runtime.
     */
    platform: string;
    /** The local port where the WebDriver server should be exposed by the tunnel. */
    port: string;
    /**
     * The protocol (e.g., 'http') that a WebDriver client should use to access the service provided by the tunnel.
     */
    protocol: string;
    /** The URL of a proxy server for the tunnel to go through. Only the hostname, port, and auth are used. */
    proxy: string;
    /** A unique identifier for the newly created tunnel. */
    tunnelId: string;
    /** The URL where the tunnel software can be downloaded. */
    url: string;
    /** Whether or not to tell the tunnel to provide verbose logging output. */
    verbose: boolean;
    protected _startTask: Task<any>;
    protected _stopTask: Promise<number>;
    protected _handle: Handle;
    protected _process: ChildProcess;
    protected _state: 'stopped' | 'starting' | 'running' | 'stopping';
    /**
     * The URL that a WebDriver client should used to interact with this service.
     */
    readonly clientUrl: string;
    /**
     * A map of additional capabilities that need to be sent to the provider when a new session is being created.
     */
    readonly extraCapabilities: Object;
    /**
     * Whether or not the tunnel software has already been downloaded.
     */
    readonly isDownloaded: boolean;
    /**
     * Whether or not the tunnel is currently running.
     */
    readonly isRunning: boolean;
    /**
     * Whether or not the tunnel is currently starting up.
     */
    readonly isStarting: boolean;
    /**
     * Whether or not the tunnel is currently stopping.
     */
    readonly isStopping: boolean;
    /**
     * Downloads and extracts the tunnel software if it is not already downloaded.
     *
     * This method can be extended by implementations to perform any necessary post-processing, such as setting
     * appropriate file permissions on the downloaded executable.
     *
     * @param forceDownload Force downloading the software even if it already has been downloaded.
     * @returns A promise that resolves once the download and extraction process has completed.
     */
    download(forceDownload?: boolean): Task<any>;
    protected _downloadFile(url: string, proxy: string, options?: DownloadOptions): Task<any>;
    /**
     * Called with the response after a file download has completed
     */
    protected _postDownloadFile(data: Buffer, options?: DownloadOptions): Promise<void>;
    /**
     * Creates the list of command-line arguments to be passed to the spawned tunnel. Implementations should
     * override this method to provide the appropriate command-line arguments.
     *
     * Arguments passed to {@link module:digdug/Tunnel#_makeChild} will be passed as-is to this method.
     *
     * @returns A list of command-line arguments.
     */
    protected _makeArgs(...values: string[]): string[];
    /**
     * Creates a newly spawned child process for the tunnel software. Implementations should call this method to
     * create the tunnel process.
     *
     * Arguments passed to this method will be passed as-is to {@link module:digdug/Tunnel#_makeArgs} and
     * {@link module:digdug/Tunnel#_makeOptions}.
     *
     * @returns An object containing a newly spawned Process and a Deferred that will be resolved once the tunnel has started
     * successfully.
     */
    protected _makeChild(executor: ChildExecutor, ...values: string[]): Task<any>;
    /**
     * Creates the set of options to use when spawning the tunnel process. Implementations should override this
     * method to provide the appropriate options for the tunnel software.
     *
     * Arguments passed to {@link module:digdug/Tunnel#_makeChild} will be passed as-is to this method.
     *
     * @returns A set of options matching those provided to Node.js {@link module:child_process.spawn}.
     */
    protected _makeOptions(...values: string[]): {
        env: any;
    };
    /**
     * Sends information about a job to the tunnel provider.
     *
     * @param jobId The job to send data about. This is usually a session ID.
     * @param data Data to send to the tunnel provider about the job.
     * @returns A promise that resolves once the job state request is complete.
     */
    sendJobState(jobId: string, data: JobState): Task<void>;
    /**
     * Starts the tunnel, automatically downloading dependencies if necessary.
     *
     * @returns A promise that resolves once the tunnel has been established.
     */
    start(): Task<any>;
    /**
     * This method provides the implementation that actually starts the tunnel and any other logic for emitting
     * events on the Tunnel based on data passed by the tunnel software.
     *
     * The default implementation that assumes the tunnel is ready for use once the child process has written to
     * `stdout` or `stderr`. This method should be reimplemented by other tunnel launchers to implement correct
     * launch detection logic.
     *
     * @returns An object containing a reference to the child process, and a Deferred that is resolved once the tunnel is
     * ready for use. Normally this will be the object returned from a call to `Tunnel#_makeChild`.
     */
    protected _start(executor: ChildExecutor): Task<any>;
    /**
     * Stops the tunnel.
     *
     * @returns A promise that resolves to the exit code for the tunnel once it has been terminated.
     */
    stop(): Promise<number>;
    /**
     * This method provides the implementation that actually stops the tunnel.
     *
     * The default implementation that assumes the tunnel has been closed once the child process has exited. This
     * method should be reimplemented by other tunnel launchers to implement correct shutdown logic, if necessary.
     *
     * @returns A promise that resolves once the tunnel has shut down.
     */
    protected _stop(): Promise<number>;
    /**
     * Get a list of environments available on the service.
     *
     * This method should be overridden and use a specific implementation that returns normalized
     * environments from the service. E.g.
     *
     * {
     *     browserName: 'firefox',
     *     version: '12',
     *     platform: 'windows',
     *     descriptor: { <original returned environment> }
     * }
     *
     * @returns An object containing the response and helper functions
     */
    getEnvironments(): Task<NormalizedEnvironment[]>;
    /**
     * Normalizes a specific Tunnel environment descriptor to a general form. To be overriden by a child implementation.
     *
     * @param environment an environment descriptor specific to the Tunnel
     * @returns a normalized environment
     */
    protected _normalizeEnvironment(environment: Object): NormalizedEnvironment;
}
export interface IOEvent extends EventTargettedObject<Tunnel> {
    readonly type: 'stdout' | 'stderr';
    readonly data: string;
}
export interface StatusEvent extends EventTargettedObject<Tunnel> {
    readonly type: 'status';
    readonly status: string;
}
export interface DownloadProgressEvent extends EventTargettedObject<Tunnel> {
    readonly type: 'downloadprogress';
    readonly url: string;
    readonly total: number;
    readonly received: number;
}
export interface ChildExecutor {
    (child: ChildProcess, resolve: () => void, reject: (reason?: any) => void): Handle | void;
}
export interface DownloadOptions {
    directory: string;
    proxy: string;
    url: string;
}
export interface NormalizedEnvironment {
    browserName: string;
    browserVersion?: string;
    descriptor: Object;
    platform: string;
    platformName?: string;
    platformVersion?: string;
    version: string;
    intern: {
        platform: string;
        browserName: string;
        version: string;
    };
}
export interface TunnelProperties extends DownloadOptions {
    architecture: string;
    auth: string;
    accessKey: string;
    executable: string;
    hostname: string;
    pathname: string;
    platform: string;
    port: string;
    protocol: string;
    tunnelId: string;
    username: string;
    verbose: boolean;
}
export declare type TunnelOptions = Partial<TunnelProperties>;
export interface Events extends BaseEventedEvents {
    (type: 'stderr' | 'stdout', listener: (event: IOEvent) => void): Handle;
    (type: 'status', listener: (event: StatusEvent) => void): Handle;
    (type: 'downloadprogress', listener: (event: DownloadProgressEvent) => void): Handle;
    (type: string, listener: (event: EventObject) => void): Handle;
}
