"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lang_1 = require("@dojo/core/lang");
var fs_1 = require("fs");
var path_1 = require("path");
/**
 * Attaches an event to a Node.js EventEmitter and returns a handle for removing the listener later.
 *
 * @param emitter A Node.js EventEmitter object.
 * @param event The name of the event to listen for.
 * @param listener The event listener that will be invoked when the event occurs.
 * @returns A remove handle.
 */
function on(emitter, event, listener) {
    emitter.on(event, listener);
    return lang_1.createHandle(function () { return emitter.removeListener(event, listener); });
}
exports.on = on;
/**
 * Returns true if a file or directory exists
 *
 * @param filename
 * @returns true if filename exists, false otherwise
 */
function fileExists(filename) {
    try {
        fs_1.statSync(filename);
        return true;
    }
    catch (error) {
        return false;
    }
}
exports.fileExists = fileExists;
/**
 * Writes data to a file.
 *
 * The file's parent directories will be created if they do not already exist.
 *
 * @param data
 * @param filename
 * @returns A Promise that resolves when the file has been written
 */
function writeFile(data, filename) {
    return new Promise(function (resolve, reject) {
        function mkdirp(dir) {
            if (!dir) {
                return;
            }
            try {
                fs_1.mkdirSync(dir);
            }
            catch (error) {
                // A parent directory didn't exist, create it
                if (error.code === 'ENOENT') {
                    mkdirp(path_1.dirname(dir));
                    mkdirp(dir);
                }
                else {
                    if (!fs_1.statSync(dir).isDirectory()) {
                        throw error;
                    }
                }
            }
        }
        mkdirp(path_1.dirname(filename));
        fs_1.writeFile(filename, data, function (error) {
            if (error) {
                reject(error);
            }
            else {
                resolve();
            }
        });
    });
}
exports.writeFile = writeFile;
//# sourceMappingURL=util.js.map